#!/bin/sh
mknod /dev/switch c 240 0

# configure classification engine for the following functionality:
# - detect LLDP discovery frames that are sent by INCAIP2 via CPU port into
#   the switch.
#   Frames are identified by
#   - their destination mac address 01:80:C2:00:00:0E(multicast), parser I used.
#   Byte position count for the following fields starts at L2 start (pos 0).
#   The 3 byte OUI field starting at byte position 42 must be equal to 0x00120F.
#   Furthermore the 1 byte subtype field at byte pos. 45 must be equal to 0x01.
# - Determine output port (LAN or PC) by checking 2 byte offset field at pos 49.
#   In case of Offset==0x0002 LAN port is used as redirection port
#   In case of Offset==0x0004 PC port is used as redirection port
#   Combined classification based on parser I and II is used for this .



# CPU Port key configuration
# Ikey0...2 = 0 -> L2 DST Addr
# Ikey3     = 1 -> Port rule match res. parser II for 2 byte offset/MAU (offset 0,1),
#                  OUI (3 bytes, upper 2 bytes -> offset 3) and
#                  subtype (1 byte subtype + lower OUI byte -> offset 2)
# Ikey4...7   = 0 -> don't care
# ./switch_control set_port_key cpu 0xkex7...0
# ./switch_control set_port_key 2 0x00001000
./switch_control set_port_key 2 4096


# set offset for key3 match with LLDP offset field (offset 0 and 1) and LLDP OUI
# (offset2 and 3)  and subtype (offset 3)
# ./switch_control set_port_offset cpu offset0=offset1=49 offset2=44 offset3=42 (each 8 bit)
#                                      offset7=0 offset6=0 offset5=0 offset4=0
# ./switch_control set_port_offset cpu offset3..0 offset7...4
# ./switch_control set_port_offset cpu 0x2A2C3131  0x00000000
./switch_control set_port_offset 2 707539249 0


# ./switch_control set_port_rule cpu  rule_31_0 rule_63_32 rule_95_64 rule_127_96
# ./switch_control set_port_rule 2 0x00040002 0x00120F01 0x00000000 0x00000000
./switch_control set_port_rule 2  262146 1183489 0 0
# ./switch_control set_port_mask cpu  mask_31_0 mask_63_32 mask_95_64 mask_127_96
# ./switch_control set_port_mask 2 0x00000000 0x00000000 0xFFFFFFFF 0xFFFFFFFF
./switch_control set_port_mask 2  0 0 4294967295 4294967295

# set_flow_pattern flow_id key_31_0 key_63_32 key_95_64 key_127_96
#                          mask_31_0 mask_63_32 mask_95_64 mask_127_96
# set_flow_pattern 0 0xC200000E 0x000E0180 0x00000000 0x00000000
#                     0x00000000 0xFFF00000 0xFFFFFFFF 0xFFFFFFFF
./switch_control set_flow_pattern 0 3254779918 917888 0 0 0 4293918720 4294967295 4294967295
# set_flow_pattern 1 0xC200000E 0x000D0180 0x00000000 0x00000000
#                    0x00000000 0xFFF00000 0xFFFFFFFF 0xFFFFFFFF
./switch_control set_flow_pattern 1 3254779918 852352 0 0 0 4293918720 4294967295 4294967295

# configure flow action params for redirection to PC port  -> flow 0
./switch_control set_flow_action_par 0 1 2
# configure flow action params for redirection to LAN port -> flow 1
./switch_control set_flow_action_par 1 1 1


################################################################################
# for reference board usage LAN and PC port assignments have to be interchanged
################################################################################
#./switch_control set_flow_action_par 0 1 1
#./switch_control set_flow_action_par 1 1 2




