/**************************************************************************
 **   FILE NAME		: ifx_os.c
 **   PROJECT			:
 **   MODULES	      : OS Interface for all IFX modules
 **   SRC VERSION		: V1.0
 **
 **   DATE				: 2-11-2004
 **   AUTHOR			: Hari
 **   DESCRIPTION		:	
 **   FUNCTIONS		:	
 **   COMPILER			:
 **   REFERENCE		:
 **							
 **   COPYRIGHT		: Infineon Technologies AG 2003-2004

 **  Version Control Section  **
 **   $Author$
 **   $Date$
 **   $Revisions$
 **   $Log$       Revision history
***********************************************************************/
/* All Standard Header files included in ifx_os.h */

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include "ifx_common_defs.h"
//#include "ifx_lldp_debug.h"
#include "ifx_os.h"

#define IFX_OS_FIFO_PERM 0666

#define IFX_OS_MX_BUFF 100
#define IFX_OS_IP_SIZE 20

char8 vcOsModId = 0;



/* Functions for FIFO Operations */

/* The OS library doesn't provide much of debug information.
 * The upper layer applications already have debugs in place
 * for error conditions.  It will increase the number of debugs
 * if they are put in both places.
 */

/******************************************************************
 *  Function Name    : IFX_OS_CreateFifo
 *  Description      : This function creates a FIFO with a given name
 *  Input Values     : Pointer to the name of the FIFO
 *  Output Values    : None
 *  Return Value     : Descriptor to the FIFO
 *  Notes            :
 ******************************************************************/
PUBLIC int32 
IFX_OS_CreateFifo(IN uchar8 * pucName)
{
   int32 iRet = 0;
   /* Check for the existence of FIFO */
   if (access(pucName, F_OK) == -1)
   {
      iRet = mkfifo(pucName, IFX_OS_FIFO_PERM);
   }/*end of if*/
   return iRet;
}

/******************************************************************
 *  Function Name    : IFX_OS_OpenFifo
 *  Description      : This function opens a FIFO with a given name
 *                     If Fifo doesnt exist, it is created first
 *  Input Values     : Pointer to the name of the FIFO, Flags
 *  Output Values    : None
 *  Return Value     : Descriptor to the FIFO
 *  Notes            :
 ******************************************************************/
PUBLIC int32
IFX_OS_OpenFifo(IN uchar8 * pucName, IN int32 iFlags)
{
   int32 iRet = 0;
   /* Check for the existence of FIFO */
   if (access(pucName, F_OK) == -1)
   {
      iRet = mkfifo(pucName, IFX_OS_FIFO_PERM);
      if (iRet < 0)
      {
         return iRet;
      }
   }/*end of if*/
   iRet = open(pucName, iFlags);
   return iRet;
}


/* C Library Functions for String Operations */

/******************************************************************************
 * Function Name: IFX_OS_Memcmp
 * Description  : Compare characters in two buffers
 * Input Values : pPtr1  - Pointer to first buffer
 *                pPtr2  - Pointer to second buffer
 *                uiSize - Number of characters to copy
 * Output Values:
 * Return Value : < 0 - buf1 less than buf2
 *                0   - buf1 identical to buf2
 *                > 0 - buf1 greater than buf2
 * Notes        :
 *****************************************************************************/
PUBLIC int32 
IFX_OS_Memcmp(IN const void * pPtr1, 
              IN const void * pPtr2,
              IN uint32 uiSize)
{
   if (!uiSize) return 0;
   while ((--uiSize != 0) && (*(char8*)pPtr1 == *(char8*)pPtr2)) 
   {
      pPtr1 = (char8*)pPtr1 + 1;
      pPtr2 = (char8*)pPtr2 + 1;
   }
   return (*((uchar8*)pPtr1) - *((uchar8*)pPtr2));
}

/******************************************************************************
 * Function Name: IFX_OS_Strlen
 * Description  : Get the length of a string
 * Input Values : pcPtr  - Pointer to buffer to copy from
 * Output Values:
 * Return Value : The number of characters in string
 * Notes        : No return value is reserved to indicate an error
 *****************************************************************************/
PUBLIC int32 
IFX_OS_Strlen(IN const char8 * pcPtr)
{
   int32 iRc = -1;
   do 
   {
      iRc++;
   } while (*pcPtr++);
   return iRc;
}

/******************************************************************************
 * Macro Name   : IFX_OS_Strcpy
 * Description  : Copy a string
 * Input Values : pcSrc  - Pointer to string buffer to copy from
 * Output Values: pcDst  - Pointer to destination string buffer
 * Return Value : The number of characters in string
 * Notes        : Pointer to destination string buffer
 *****************************************************************************/
PUBLIC char8 * 
IFX_OS_Strcpy(IN char8 * pcDst, 
              IN const char8 * pcSrc)
{
   char8 * cRc = pcDst;
   do 
   {
      *pcDst = *pcSrc++;
   } while (*pcDst++);
   return cRc;
}

/******************System V Semphore operations **********/

/************************************************************************
 *  Function Name: IFX_OS_LOCK
 *  Description  : This function sets the Netmask to a particular
 *                 Interface 
 *  Input Values : Pointer to the Netmask to be added
 *               : Interface Name
 *  Output Values: None
 *  Return Value : 0 or -1
 *  Notes        :
 ************************************************************************/
/* Lock Semphore*/
PUBLIC int32 IFX_OS_LOCK(IN int32 iSemId)
{
  struct sembuf xWait ={0,-1,SEM_UNDO};
  
  return IFX_OS_SemOp(iSemId,&xWait,1);

  
}
/************************************************************************
 *  Function Name: IFX_OS_UNLOCK
 *  Description  : This function sets the Netmask to a particular
 *                 Interface 
 *  Input Values : Pointer to the Netmask to be added
 *               : Interface Name
 *  Output Values: None
 *  Return Value : 0 or -1
 *  Notes        :
 ************************************************************************/

/*Unlock Semphore*/
PUBLIC int32 IFX_OS_UNLOCK(IN int32 iSemId)
{
   struct sembuf xPost = {0,1,SEM_UNDO};
   return IFX_OS_SemOp(iSemId,&xPost,1);
       
}

/************** File Operations ****************/
