/*******************************************************************
 *
 * OpenLLDP RX Statemachine Header
 *
 * Licensed under a dual GPL/Proprietary license.  
 * See LICENSE file for more info.
 * 
 * File: lldp_sm_rx.h
 * 
 * Authors: Terry Simons (terry.simons@gmail.com)
 *
 *******************************************************************/

#ifndef LLDP_RX_SM_H
#define LLDP_RX_SM_H

struct lldp_port;

// States from the receive state machine diagram (IEEE 802.1AB 10.5.5.3)
typedef enum {
	LLDP_WAIT_PORT_OPERATIONAL 	= 4,
	DELETE_AGED_INFO		= 5,
	RX_LLDP_INITIALIZE		= 6,
	RX_WAIT_FOR_FRAME		= 7,
	RX_FRAME			= 8,
	DELETE_INFO			= 9,
	UPDATE_INFO			= 10
} lldp_rx_states;

/* Utility functions  */

/* This utility function is used to change the state of the receive FSM */
void rxChangeToState(struct lldp_port *lldp_port, uint8 state);

/* This internal function is invoked to trigger the receive FSM */
void rxStatemachineRun(struct lldp_port *lldp_port);

#if 0
void rxBadFrameInfo(uint8 badFrameCount);
char *rxStateFromID(uint8 state);
#endif

/* Receive FSM functions */

/* This state handler function handles LLDP_WAIT_PORT_OPERATIONAL state */
void rx_do_lldp_wait_port_operational(struct lldp_port *lldp_port);

/* This state handler function handles DELETE_AGED_INFO state */
void rx_do_delete_aged_info(struct lldp_port *lldp_port);

/* This state handler function handles RX_LLDP_INITIALIZE state */
void rx_do_rx_lldp_initialize(struct lldp_port *lldp_port);

/* This state handler function handles RX_WAIT_FOR_FRAME state */
void rx_do_rx_wait_for_frame(struct lldp_port *lldp_port);

/* This state handler function handles RX_FRAME state */
void rx_do_rx_frame(struct lldp_port *port);

/* This state handler function handles DELETE_INFO state */
void rx_do_rx_delete_info(struct lldp_port *lldp_port);

/* This state handler function handles UPDATE_INFO state */
void rx_do_rx_update_info(struct lldp_port *lldp_port);

#endif /* LLDP_RX_SM_H */
