/**************************************************************************
**   FILE NAME		: ifx_lldp_local.c
**   PROJECT		: Inca-Ip
**   MODULES	    : LLDP
**   SRC VERSION	: V1.0
**
**   DATE			:12-04-2007
**   AUTHOR			:Sanat. B,Pavan GM
**   DESCRIPTION	:Source file 	
**   COMPILER		:GCC
**   REFERENCE		:Coding Guideline Version 2.0
**							
**   COPYRIGHT      :Copyright © 2004
**                   Infineon Technologies AG
**                   St. Martin Strasse 53; 81669 München, Germany
**   DESCLAIMER     :Any use of this Software is subject to the conclusion 
**                   of a respective License Agreement.
**                   Without such a License Agreement no rights to the 
**         			 Software are granted. 
**
**  Version Control Section 
**   $Author$
**   $Date$
**   $Revisions$
**   $Log$       Revision history
***********************************************************************/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <netinet/in.h>
#include <sys/ioctl.h>
#include <linux/socket.h>
#include <linux/if_packet.h>
#include <errno.h>
#include <net/if.h>

#ifdef __LINUX__
#include <malloc.h>
#endif /* __LINUX__ */

#include "ifx_common_defs.h"
#include "ifx_lldp_config.h"
#include "ifx_lldp_neighbor.h"
#include "ifx_lldp_tx_sm.h"
#include "ifx_lldp_rx_sm.h"
#include "ifx_lldp_mib_if.h"
#include "ifx_lldp_cm_defs.h"
#include "ifx_lldp_cm.h"
#include "ifx_lldp_rx_module.h"
#include "ifx_lldp_tx_module.h"
#include "ifx_lldp_tlv.h"
#include "ifx_lldp_port.h"
#include "ifx_lldp_local.h"

/* External variables */
EXTERN fd_set readfds;
EXTERN struct lldp_port *vpxlldp_port;
EXTERN void IFX_LLDP_FdSet(int32 soc);//in main.c

int32 viNumPorts = 0;
boolean bLldpduStorage;
boolean bModuleInited;

/*****************************************************************************
*  Function Name        :   IFX_LLDP_HandleManagerEvents
*  Description          :   This function handles events from LLDP manager Timer
                             that are read from FIFO.
*                           
*  Input Values         :   None 
*  Output Values        :   None
*  Return Value         :   0 - On Success
*                           -1 - On Failure
*  Notes                :
*****************************************************************************/
int32 IFX_LLDP_HandleManagerEvents(x_IFX_CM_Msg *pxCmMsg)
{
	PRINT("\n IN->IFX_LLDP_HandleManagerEvents \n");
	char8 uiPortArrIndex = -1;	
	uint8 i;
	switch(pxCmMsg->xlldpAgentMsg.eEventType)
	{
		case LLDP_MODULE_INIT_MSG :
		{
			/*create global port str array required for lldp agent*/
			PRINT("\nLLDP_MODULE_INIT_MSG\n");
			if (bModuleInited == TRUE)
			{
				fprintf(stdout,"\nModule Init already done!\n");
				return -1;
			}
			if(vpxlldp_port != NULL)
			{
				free(vpxlldp_port);
			}

			vpxlldp_port= ((struct lldp_port *)malloc(
			sizeof(struct lldp_port) * pxCmMsg->PortIndex));
			if(vpxlldp_port == NULL)
			{
				PRINT("\n memory not allocated");
				return LLDP_FAIL;
			}
			else
			{
				PRINT("\nMemory allocated for %d ports\n",pxCmMsg->PortIndex);
			}

			viNumPorts = pxCmMsg->PortIndex; /*Number of ports required for lldp_agent*/

			for(i=0;i<viNumPorts;i++)
			{
				if((vpxlldp_port+i) != NULL)
				{
					(vpxlldp_port+i)->if_name = NULL;
					(vpxlldp_port+i)->uiPortIndex = -1;
				}
			}
			 bLldpduStorage =pxCmMsg->bLldpduStorage;
			 /*create and attach shared memory.Done to get CM ptrs in our process  XXX*/
			IFX_LLDP_CM_SHM_Init(viNumPorts);
			/*Not required to invoke TX_RX_hanldeEvent XXX ???*/
			bModuleInited = TRUE;
			break;
		}
		case LLDP_INIT_MSG :
		{
		/* PORT_INIT->Just store intf name and port index.Socket opening would be done in PORT_ENABLED*/
		PRINT("\n LLDP_INIT_MSG\n");
		/* pxCmMsg->PortIndex - 0->PC , 1->LAN....also the array index for 
		the appropriate port st */
			if(pxCmMsg->IfrName != NULL)
			{
				//uiPortArrIndex = pxCmMsg->PortIndex;
				uiPortArrIndex = IFX_LLDP_GetArrayIndex(pxCmMsg->PortIndex);
				if(uiPortArrIndex > -1)  //entry there.port already intited
				{
					fprintf(stdout, "\n[INFO] Port already Inited\n");
				}
				else
				{
					/*New port.Put this in an unused entry*/ 
					PRINT("\nIntializing new port\n");
					/*Initialize only for the first time*/
					for(i=0;i<viNumPorts;i++)
					{
						if((vpxlldp_port+i)->if_name == NULL)
						{
							/*store Port Index and Intf Name*/
							(vpxlldp_port+i)->if_name = malloc(IFX_LLDP_IF_NAMESIZE);
							memcpy((vpxlldp_port+i)->if_name,pxCmMsg->IfrName,IFX_LLDP_IF_NAMESIZE);
							(vpxlldp_port+i)->uiPortIndex=pxCmMsg->PortIndex;
							PRINT("\nPortIndex stored in INIT = %d\n",(vpxlldp_port+i)->uiPortIndex);
							/*Initialize other variables*/	
							(vpxlldp_port+i)->portEnabled=FALSE;
							(vpxlldp_port+i)->socket=-1; 
							(vpxlldp_port+i)->rx.frame=NULL; 
							(vpxlldp_port+i)->tx.frame=NULL;
							PRINT("\n HandlemanagerEvents LLDP_INIT_MSG adminStatus = %d\n",(vpxlldp_port+i)->adminStatus);
							uiPortArrIndex = i;//Index to be sent to Handle_event 
							break;
						}
					}
				}
			}
			else
			{
				PRINT("\nInvalid Intf Name sent by LLDP Manager\n");
				break;
			}
			/*Now Initialize TX RX*/
			LLdp_tx_HandleEvent(uiPortArrIndex,&(pxCmMsg->xlldpAgentMsg));
			LLdp_rx_HandleEvent(uiPortArrIndex,&(pxCmMsg->xlldpAgentMsg));
			break;
		}
		case LLDP_PORT_ENABLED_MSG:
		{

			PRINT("\n LLDP_PORT_ENABLED_MSG\n");
			uint8 uiPortPresent=0;
			//uiPortArrIndex=pxCmMsg->PortIndex;//Index to be enabled

			/* get array index from port index to be enabled */
			uiPortArrIndex=IFX_LLDP_GetArrayIndex(pxCmMsg->PortIndex);
			if(uiPortArrIndex < 0)
			{
				fprintf(stdout, "\n[ERROR] Port not inited\n");
				break;
			}

			(vpxlldp_port+uiPortArrIndex)->portEnabled=TRUE;

			/* pxCmMsg->PortIndex - 0->PC , 1->LAN....also the array index for the 
			appropriate port st */
			PRINT("\n**PORT Enable Msg for %s Intf having %d Port Index**\n",
			(vpxlldp_port+uiPortArrIndex)->if_name,
			(vpxlldp_port+uiPortArrIndex)->uiPortIndex);
			/*Check if Intf name already present and initialized.*/
			for(i=0;i<viNumPorts;i++)
			{
				if((vpxlldp_port+i)->if_name != NULL)
				{
					if((strcmp((vpxlldp_port+i)->if_name,(vpxlldp_port+uiPortArrIndex)->if_name) == 0) && 
						((vpxlldp_port+i)->socket > -1))
					{
						PRINT("\nEnabling a port index for interface already initialized\n");
						uiPortPresent=1;
						(vpxlldp_port+(uiPortArrIndex))->socket=(vpxlldp_port+i)->socket; 
						(vpxlldp_port+(uiPortArrIndex))->if_index=(vpxlldp_port+i)->if_index;
						memcpy(&(vpxlldp_port+(uiPortArrIndex))->source_mac[0],&(vpxlldp_port+i)->source_mac[0],6);
						(vpxlldp_port+(uiPortArrIndex))->mtu= 			 (vpxlldp_port+i)->mtu;
						memcpy(&(vpxlldp_port+(uiPortArrIndex))->source_ipaddr[0],&(vpxlldp_port+i)->source_ipaddr[0],4);
						/*Malloc RX TX FRAME only if this is being enabled first time*/
						/*In case of disbling and then enabling the port it would already be malloced*/ 
						if((vpxlldp_port+uiPortArrIndex)->rx.frame == NULL)
						{
							(vpxlldp_port+uiPortArrIndex)->rx.frame = malloc((vpxlldp_port+uiPortArrIndex)->mtu);
							bzero(&(vpxlldp_port+uiPortArrIndex)->rx.frame[0], (vpxlldp_port+uiPortArrIndex)->mtu);
						}
						if((vpxlldp_port+uiPortArrIndex)->tx.frame == NULL)
						{
							(vpxlldp_port+uiPortArrIndex)->tx.frame = malloc((vpxlldp_port+uiPortArrIndex)->mtu);
							bzero(&(vpxlldp_port+uiPortArrIndex)->tx.frame[0], (vpxlldp_port+uiPortArrIndex)->mtu);
						}
						//(vpxlldp_port+uiPortArrIndex)->medTransmitEnabled=(vpxlldp_port+i)->medTransmitEnabled;
						break;
					}
				}
			}
			 /*new port*/
			if( uiPortPresent == 0)
			{
				PRINT("\nEnabling a new port index for an interface \n");

				/*create socket interface for this port.Get the ifrindex,mac addr here.code from open lldp*/
				if(initializeLLDP(vpxlldp_port+(uiPortArrIndex)) != 0)
				{
					PRINT("\n IFX_LLDP_HandleManagerEvents -> initializeLLDP : failed\n");
					return -1;
				}

				/*Add soc to ReaFdset*/
				IFX_LLDP_FdSet((vpxlldp_port+(uiPortArrIndex))->socket);
			}

			LLdp_tx_HandleEvent(uiPortArrIndex,&(pxCmMsg->xlldpAgentMsg));
			LLdp_rx_HandleEvent(uiPortArrIndex,&(pxCmMsg->xlldpAgentMsg));
		}
		break;
		
		case LLDP_PORT_DISABLED_MSG:
		{
			PRINT("\n LLDP_PORT_DISABLED_MSG\n");
			uiPortArrIndex=IFX_LLDP_GetArrayIndex(pxCmMsg->PortIndex);//Get array index from port index to be disbled
			if(uiPortArrIndex < 0)
			{
				fprintf(stdout, "\nLLDP_PORT_DISABLED_MSG: Port not Inited\n");
				break;
			}
			(vpxlldp_port+uiPortArrIndex)->portEnabled = FALSE;

			char* disableif=(vpxlldp_port+uiPortArrIndex)->if_name;
			PRINT("\ndisableintf %s index %d \n",disableif,uiPortArrIndex);
			// OP causes seg fault PRINT("\n(vpxlldp_port+i)->if_name) %s \n",(vpxlldp_port+i)->if_name);
			for(i = 0;i<viNumPorts;i++)
			{
				PRINT("\nbefore strcmp\n");
				if((vpxlldp_port+i)->if_name != NULL)
				{
					/*the third condition is because->both PC/LAN are enabled.Disable PC[skt=-1].Now disable LAN.
					*then check if other port is using it or not[skt!=-1].If using it then just initialize the current port
					*to -1 else no one is using it so close the port*/
					if((strcmp(disableif,(vpxlldp_port+i)->if_name)==0) && (i!=uiPortArrIndex) && ((vpxlldp_port+i)->socket)!=-1)
					{
						//there is one more port index using the same intf. 
						PRINT("\nInit socket = -1\n");
						(vpxlldp_port+uiPortArrIndex)->socket=-1;//someone else using this fd.so dont close
						(vpxlldp_port+uiPortArrIndex)->mtu= 0;
						break;
					}
				}
			}
			if(i == viNumPorts)
			{
				PRINT("\nClosing skt\n");
				FD_CLR((vpxlldp_port+uiPortArrIndex)->socket,&readfds );
				close((vpxlldp_port+uiPortArrIndex)->socket);//this is the only one using.so close it.
				(vpxlldp_port+uiPortArrIndex)->socket=-1;//someone else using this fd.so dont close
				(vpxlldp_port+uiPortArrIndex)->mtu= 0;
				/*will be malloced again in PORT_ENABLE->sktinitialize()*/
				free((vpxlldp_port+uiPortArrIndex)->rx.frame);
				(vpxlldp_port+uiPortArrIndex)->rx.frame=NULL;
				free((vpxlldp_port+uiPortArrIndex)->tx.frame);
				(vpxlldp_port+uiPortArrIndex)->tx.frame=NULL;
			}

			LLdp_tx_HandleEvent(uiPortArrIndex,&(pxCmMsg->xlldpAgentMsg));
			LLdp_rx_HandleEvent(uiPortArrIndex,&(pxCmMsg->xlldpAgentMsg));
		}
		break;
		case LLDP_CONFIG_MSG :
		{
			PRINT("\nLLDP_CONFIG_MSG\n");
			uiPortArrIndex = IFX_LLDP_GetArrayIndex(pxCmMsg->PortIndex);
			if(uiPortArrIndex < 0)
			{
				fprintf(stdout, "\nLLDP_CONFIG_MSGPort not Inited\n");
				break;
			}

			/*all config data ll be updated in CM, in the CM_API itself before coming here.*/
			/*Just pass to TX RX..should invoke both blindly or check adminstatus and invoke accordingly XXX ???*/
			struct LldpConfigMib Config;
			//update adminstatus in lldp_port  
			getMibData(localConfig,(vpxlldp_port+uiPortArrIndex)->uiPortIndex,NULL,(void *)&Config);

			enum portAdminStatus eTempStatus = (vpxlldp_port+uiPortArrIndex)->adminStatus;

			(vpxlldp_port+uiPortArrIndex)->adminStatus=Config.adminStatus;
			PRINT("\n HandlemanagerEvents LLDP_CONFIG_MSG (vpxlldp_port+uiPortArrIndex)->adminStatus = %d\n",(vpxlldp_port+uiPortArrIndex)->adminStatus);

			switch((vpxlldp_port+uiPortArrIndex)->adminStatus)
			{
				case disabled:
					PRINT("\nCASE : DISABLED\n");
					switch(eTempStatus)
					{
						case enabledTxOnly:
						LLdp_tx_HandleEvent(uiPortArrIndex,&(pxCmMsg->xlldpAgentMsg));
						break;
						case enabledRxOnly:
						LLdp_rx_HandleEvent(uiPortArrIndex,&(pxCmMsg->xlldpAgentMsg));
						break;
						case enabledRxTx:
						LLdp_tx_HandleEvent(uiPortArrIndex,&(pxCmMsg->xlldpAgentMsg));
						LLdp_rx_HandleEvent(uiPortArrIndex,&(pxCmMsg->xlldpAgentMsg));
						break;
						default:
						break;
					}
				break;
				case enabledTxOnly:
				case enabledRxOnly:
				case enabledRxTx:
					LLdp_tx_HandleEvent(uiPortArrIndex,&(pxCmMsg->xlldpAgentMsg));
					LLdp_rx_HandleEvent(uiPortArrIndex,&(pxCmMsg->xlldpAgentMsg));
				break;
			}
			break;
		}
		case LLDP_LOCAL_MIB_CHANGE_MSG :
		{
			PRINT("\nLLDP_LOCAL_MIB_CHANGE_MSG\n");
			PRINT("\nFIFO PORT INDEX GOT = %d\n",pxCmMsg->PortIndex);
			uiPortArrIndex = IFX_LLDP_GetArrayIndex(pxCmMsg->PortIndex);
			if(uiPortArrIndex < 0)
			{
				fprintf(stdout, "\nLLDP_LOCAL_MIB_CHANGE_MSG Port not Inited\n");
				break;
			}

			LLdp_tx_HandleEvent(uiPortArrIndex,&(pxCmMsg->xlldpAgentMsg));
			break;
		}
		case LLDP_SHUTDOWN_MSG :
		{
			PRINT("\nLLDP_SHUTDOWN_MSG\n");
			cleanupLLDP(pxCmMsg);
			IFX_LLDP_CM_SHM_Shut(); //delete shmem
			exit(0); //exit from the process
			break;
		}
#if 0
		case LLDP_DISABLE_MSG :
		cleanupLLDP(pxCmMsg);
		break;
#endif
		default :
		{
			PRINT("\n IFX_LLDP_HandleManagerEvents : wrong  Msg ID\n");
		}
	}
	return 0;
}

/*****************************************************************************
*  Function Name        :   IFX_LLDP_HandleLocalEvents
*  Description          :   This function handles events from LLDP manager Timer
                             that are read from FIFO.
*                           
*  Input Values         :   None 
*  Output Values        :   None
*  Return Value         :   0 - On Success
*                           -1 - On Failure
*  Notes                :
*****************************************************************************/
int32 IFX_LLDP_HandleLocalEvents(x_IFX_CM_Msg *pxCmMsg)
{
	switch(pxCmMsg->eModType)/*from which module*/
	{
	case IFX_LLDP_TIMER:
	{
		switch(pxCmMsg->xlldpAgentMsg.eEventType)
		{
           case LLDP_REMOTE_TIMER_MSG :
		   {
				 PRINT("\nRemote Timeout message received\n");
			   LLdp_rx_HandleEvent(
           IFX_LLDP_GetArrayIndex(pxCmMsg->xlldpAgentMsg.ucPort),
           &(pxCmMsg->xlldpAgentMsg)); 
			   break;
		   }
		   case LLDP_LOCAL_TIMER_MSG :
		   {
				 PRINT("\nLocal Timeout message received\n");
			   LLdp_tx_HandleEvent(
           IFX_LLDP_GetArrayIndex(pxCmMsg->xlldpAgentMsg.ucPort),
           &(pxCmMsg->xlldpAgentMsg));			   
			   break;
		   }
		   default :
		   {
			   PRINT("\n IFX_LLDP_HandleLocalEvents : wrong Timer Msg ID\n");
			   break;
		   }

		}
		break;
	}
	case IFX_LLDP_LLDP_MANAGER:
	{
		IFX_LLDP_HandleManagerEvents(pxCmMsg);
		break;
	}
	default :
		PRINT("\n IFX_LLDP_HandleLocalEvents : worng module ID\n");
	}

	return 0;
}

/*****************************************************************************
*  Function Name        :   initializeLLDP
*  Description          :   This function handles the port initialization part
*  Input Values         :   None
*  Output Values        :   None
*  Return Value         :   0 - On Success
*                           -1 - On Failure
*  Notes                :
*****************************************************************************/

int32 initializeLLDP(struct lldp_port *lldp_port)
{
  PRINT("\nIN->initializeLLDP\n");
  int32 if_index = 0;

  /* We need to initialize an LLDP port-per interface */
  /* "lldp_port" will be changed to point at the interface currently 
     being serviced */
      
  PRINT("\nCurrent Ifr_name=%s\n\n",lldp_port->if_name);

  //if(if_nametoindex(if_index, &lldp_port->if_name[0]) != NULL)
  if((if_index=if_nametoindex(lldp_port->if_name)) >= 0)

	{
	  PRINT("\n\nInterface ID for %s =  %d \n\n",lldp_port->if_name,if_index);
	  
    // Store the interface index in the data structure
	  lldp_port->if_index = if_index;
	      
	  /*Now Initialize the socket for this interface as we ve valid i/f name and index*/
	  if(socketInitializeLLDP(lldp_port) != 0) 
    {
      PRINT("\n\nProblem initializing socket for %s\n\n", lldp_port->if_name);
	  }
	      
	} 
  else // Otherwise set the data structure values to sane non-random values...
	{
	  /* Initialize the globals */
	  PRINT("\n\nin the else part somehow\n\n");
	  lldp_port->socket      = 0;
	  lldp_port->if_name     = NULL;
	  lldp_port->if_index    = 0;
	  lldp_port->mtu         = 0;
	  lldp_port->portEnabled = 0;
	  lldp_port->adminStatus = disabled;
      PRINT("\n\nOUT->initilaizeLLDP->lldp_port->adminStatus\n",lldp_port->adminStatus);

	  /* Initialize the rx port variables */
	  lldp_port->rx.frame    = NULL;
	  lldp_port->rx.recvsize = 0;

	  /* Initialize the tx port variables */
	  lldp_port->tx.frame    = NULL;
	  lldp_port->tx.sendsize = 0;
	}
    get_sys_desc();
    get_sys_fqdn();

  /* Initialize the TLV validators */
  initializeTLVFunctionValidators();

  PRINT("\n\nOUT->initilaizeLLDP->i/f name =%s\n",lldp_port->if_name);
  return 0;
}

/*****************************************************************************
*  Function Name        :   handle_signal
*  Description          :   This function handles sigfault
*  Input Values         :   None
*  Output Values        :   None
*  Return Value         :   0 - On Success
*                           -1 - On Failure
*  Notes                :
*****************************************************************************/
void handle_signal()
{
  PRINT("\nIN->handle_signal\n");
  x_IFX_CM_Msg xCmMsg;
  xCmMsg.xlldpAgentMsg.eEventType = LLDP_SHUTDOWN_MSG;
  IFX_LLDP_HandleManagerEvents(&xCmMsg);
}

/*****************************************************************************
*  Function Name        :   socketInitializeLLDP
*  Description          :   This function creates RAW SOCKET for a particular port
*                           and handles corresponding operations
*  Input Values         :   lldp_port
*  Output Values        :   None
*  Return Value         :   0 - On Success
*                           -1 - On Failure
*  Notes                :
*****************************************************************************/

int32 socketInitializeLLDP(struct lldp_port *lldp_port)
{
  struct ifreq ifr;
  struct sockaddr_ll sll;
  int32 retval             = 0;
  PRINT("\nIN->socketInitializeLLDP\n");
  PRINT("\nIF_NAME=%s\n",lldp_port->if_name);
  PRINT("\nIF_INDEX=%d\n",lldp_port->if_index);

  if(lldp_port->if_name == NULL)
  {
    PRINT("\n\nNULL IF_NAME..exiting\n\n");
    exit(-1);
  }

  if(!lldp_port->if_index > 0)
  {
    PRINT("\n\nINVALID IF_INDEX\n\n");
    return 0;
  }

  // Set up the raw socket for the LLDP ethertype
  lldp_port->socket = socket(PF_PACKET, SOCK_RAW, htons(0x88CC));

  if(lldp_port->socket < 0)
  {
    PRINT("\nCouldn't initialize raw socket for interface %s!\n", lldp_port->if_name);
    return 0;
  }

  // Set up the interface for binding
  sll.sll_family = PF_PACKET;
  sll.sll_ifindex = lldp_port->if_index;
  sll.sll_protocol = htons(0x88CC);
  //sll.sll_ifindex = lldp_port->if_index;
  
  // Bind the socket and the interface together to form a useable socket:
  retval = bind(lldp_port->socket, (struct sockaddr *)&sll, sizeof(struct sockaddr_ll));

  if(retval < 0)
  {
    PRINT("\nError binding raw socket to interface\n");
    return -1;
  }
  //memset(&ifr, 0x00, sizeof(ifr));

  ifr.ifr_ifindex = lldp_port->if_index;

  strncpy((char *)&ifr.ifr_name, &lldp_port->if_name[0], sizeof(ifr.ifr_name));
  PRINT("\n\nifr.ifr_name = %s\n" , ifr.ifr_name);

  if(strlen(ifr.ifr_name) == 0)
  {
    PRINT("\nifr invalid intf name length\n");
      return -1;
  }

  //Get the src mac addr
  retval = ioctl(lldp_port->socket, SIOCGIFHWADDR, &ifr);

  if(retval < 0)
  {
    fprintf(stdout, "\nError getting hardware (MAC) address for interface %s\n",
      lldp_port->if_name);
    return retval;
  }

  memcpy(&lldp_port->source_mac[0], &ifr.ifr_hwaddr.sa_data[0], 6);

  PRINT("\n\nSource MAC is: %d\n",lldp_port->source_mac[0]);

  // get interface ip address
  retval = ioctl(lldp_port->socket, SIOCGIFADDR, &ifr);

  if (retval < 0) 
  {
    fprintf(stdout, "\nError getting interface IP address for interface %s\n",
      lldp_port->if_name); 
    //return retval;
  }
  else
  {
    memcpy(&lldp_port->source_ipaddr[0], &ifr.ifr_addr.sa_data[2], 4);
  }

  memcpy(&lldp_port->source_ipaddr[0], &ifr.ifr_addr.sa_data[2], 4);
 
  //find whether the port is UP/DOWN
  retval = ioctl(lldp_port->socket, SIOCGIFFLAGS, &ifr);

  if (retval == -1)
  {
    PRINT("\nCan't get flags for interface %s \n", lldp_port->if_name);    
  }

  if ((ifr.ifr_flags & IFF_UP) == 0) 
  {
    lldp_port->portEnabled = 0;
  }
  else
    lldp_port->portEnabled = TRUE;

  // set allmulti on interface
  /* need to devise a graceful way to turn off allmulti otherwise it is left 
     on for the interface when problem is terminated */
  retval = ioctl(lldp_port->socket, SIOCGIFFLAGS, &ifr);

  if (retval == -1)
  {
    PRINT("\nCan't get flags for interface %s \n", lldp_port->if_name);    
  }

  ifr.ifr_flags |=  IFF_ALLMULTI; // allmulti on (verified via ifconfig)
  //  ifr.ifr_flags &= ~IFF_ALLMULTI; // allmulti off (I think)

  retval = ioctl(lldp_port->socket, SIOCSIFFLAGS, &ifr);
  
  if (retval == -1)
  {
    
    PRINT("\nCan't set flags for interface %s \n", lldp_port->if_name);    
  }

  // Discover MTU of our interface.
  retval = ioctl(lldp_port->socket, SIOCGIFMTU, &ifr);

  if(retval < 0) 
  {
    PRINT("\nCan't get MTU for interface %s \n", lldp_port->if_name);    
    return retval;
  }	

  /*store MTU details*/
  lldp_port->mtu = ifr.ifr_ifru.ifru_mtu;

  lldp_port->rx.frame = malloc(lldp_port->mtu);
  lldp_port->tx.frame = malloc(lldp_port->mtu);

  if(!lldp_port->rx.frame) {
    PRINT("\nUnable to malloc buffer for rx frame\n");
  } 

  if(!lldp_port->tx.frame) {
    PRINT("\nUnable to malloc buffer for tx frame\n");
  }

  bzero(&lldp_port->rx.frame[0], lldp_port->mtu);
  bzero(&lldp_port->tx.frame[0], lldp_port->mtu);

  PRINT("\n\nInterface %s MTU is %d.\n\n", lldp_port->if_name, lldp_port->mtu);

  return 0;
}

/*****************************************************************************
*  Function Name        :   cleanupLLDP
*  Description          :   This function dones the cleanup activities
*  Input Values         :   lldp_port,CM msg
*  Output Values        :   None
*  Return Value         :   0 - On Success
*                           -1 - On Failure
*  Notes                :
*****************************************************************************/

void cleanupLLDP(x_IFX_CM_Msg *pxCmMsg) 
{
  PRINT("\nIN->cleanupLLDP\n");
  int32 portindex = 0;
  switch(pxCmMsg->xlldpAgentMsg.eEventType)
  {
    case LLDP_SHUTDOWN_MSG :
    /*disable all ports*/ 
    for(portindex = 0;portindex<viNumPorts;portindex++)
    {
      
      if((vpxlldp_port+portindex)->if_name != NULL)
      {
      	LLdp_tx_HandleEvent(portindex,&(pxCmMsg->xlldpAgentMsg));
      	LLdp_rx_HandleEvent(portindex,&(pxCmMsg->xlldpAgentMsg));

        PRINT("\nCleaning up interface: '%s'\n", (vpxlldp_port+portindex)->if_name);
        socketCleanupLLDP(vpxlldp_port+portindex);
        free((vpxlldp_port+portindex)->if_name);
        (vpxlldp_port+portindex)->if_name = NULL;
      }

    }
    /*free the complete port st*/
    if(vpxlldp_port != NULL)
    {
      free(vpxlldp_port);
      vpxlldp_port = NULL;     
    }
  
    break;
    default:
    break;
  }
  PRINT("\nOUT->cleanupLLDP\n");
}

/*****************************************************************************
*  Function Name        :   socketCleanupLLDP
*  Description          :   This function dones the socket cleanup activities
*  Input Values         :   lldp_port
*  Output Values        :   None
*  Return Value         :   0 - On Success
*                           -1 - On Failure
*  Notes                :
*****************************************************************************/

void socketCleanupLLDP(struct lldp_port *lldp_port) 
{
  PRINT("\nIN->socketCleanupLLDP\n");

  if(lldp_port != NULL) 
  {
    /*Free rx frame*/
    if(lldp_port->rx.frame != NULL) 
    {   
      free(lldp_port->rx.frame);
      lldp_port->rx.frame = NULL;
    } 
    else 
    {
      PRINT("[ERROR] lldp_port->rx.frame was NULL!\n");
    }
   
    /*free tx frame*/ 
    if(lldp_port->tx.frame != NULL) 
     {
      free(lldp_port->tx.frame);
      lldp_port->tx.frame = NULL;
     } 
     else 
     {
      PRINT("[ERROR] lldp_port->tx.frame was NULL!\n");
     }
  } 
  else 
  {
      PRINT("[ERROR] lldp_port was NULL!\n");
  }
}

/*****************************************************************************
*  Function Name        :   IFX_LLDP_GetArrayIndex
*  Description          :   Returns the local array index corresponding to the
*                           port id.
*  Input Values         :   Port Index
*  Output Values        :   None.
*  Return Value         :   Local Array Index.
*  Notes                :
*****************************************************************************/
char8 IFX_LLDP_GetArrayIndex(uchar8 uiPortIndex)
{
	uchar8 i=0;
  PRINT("\nPortIndex %d\n",uiPortIndex);
		
	for(i=0;i<viNumPorts;i++)
	{
		if((vpxlldp_port+i)->uiPortIndex == uiPortIndex)
		{
			/*that port is already inited so send the index,
			may be wants to over write port info*/			
			return i;		
		}
	}
  return -1;
}

