/*******************************************************************
 *
 * OpenLLDP TX Statemachine
 *
 * Licensed under a dual GPL/Proprietary license.  
 * See LICENSE file for more info.
 * 
 * File: lldp_sm_tx.c
 * 
 * Authors: Terry Simons (terry.simons@gmail.com)
 *
 *******************************************************************/
/**************************************************************************
**   FILE NAME		: ifx_lldp_tx_sm.c
**   PROJECT		: Inca-Ip
**   MODULES	    : LLDP
**   SRC VERSION	: V1.0
**
**   DATE			:12-04-2007
**   AUTHOR			:Pavan GM
**   DESCRIPTION	:Source file 	
**   COMPILER		:GCC
**   REFERENCE		:Coding Guideline Version 2.0
**							
**   COPYRIGHT      :Copyright © 2004
**                   Infineon Technologies AG
**                   St. Martin Strasse 53; 81669 München, Germany
**   DESCLAIMER     :Any use of this Software is subject to the conclusion 
**                   of a respective License Agreement.
**                   Without such a License Agreement no rights to the 
**         			 Software are granted. 
**
**  Version Control Section 
**   $Author$
**   $Date$
**   $Revisions$
**   $Log$       Revision history
***********************************************************************/

#include <netinet/in.h>
#ifdef DEBUG_ENABLED
#include <stdio.h>
#endif

#include "ifx_common_defs.h"
#include "ifx_lldp_config.h"
#include "ifx_lldp_cm.h"
#include "ifx_lldp_tx_sm.h"
#include "ifx_lldp_tx_module.h"
#include "ifx_lldp_mib_if.h"
#include "ifx_lldp_rx_sm.h"
#include "ifx_lldp_rx_module.h"
#include "ifx_lldp_port.h"
#include "ifx_lldp_tlv.h"
#include "ifx_lldp_timer_if.h"

STATIC uint16 min(uint16 value1, uint16 value2);

/*******************************************************************************
*  Function Name    : txChangeToState
*  Description      : This function updates the current state in lldp_port st
*  Input Values     : lldp_port,current Tx state
*  Output Values    : None
*  Return Value     : None
*  Notes            :
******************************************************************************/

inline void txChangeToState(struct lldp_port *lldp_port, uint8 state)
{
  lldp_port->tx.state = state;
}

char *txStateFromID(uint8_t state)
{
  switch(state)
    {
    case TX_LLDP_INITIALIZE:
      return "TX_LLDP_INITIALIZE";
    case TX_IDLE:
      return "TX_IDLE";
    case TX_SHUTDOWN_FRAME:
      return "TX_SHUTDOWN_FRAME";
    case TX_INFO_FRAME:
      return "TX_INFO_FRAME";
    };

  return "Unknown";
}

/*******************************************************************************
*  Function Name    : txGlobalStatemachineRun
*  Description      : this fn does transition between states based on 
*		                  the satisfaction of prerequisite conditions.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
******************************************************************************/

void txGlobalStatemachineRun(struct lldp_port *lldp_port)
{
  /* Sit in TX_LLDP_INITIALIZE until the next initialization.This condition is 
     checked as port enabled can false anytime */
  PRINT("\nIN->txGlobalStatemachineRun cur state %d tx state %d\n",
    lldp_port->tx.currstate,lldp_port->tx.state);
  PRINT("\nPE = %d,  adminstatus = %d\n",
    lldp_port->portEnabled,lldp_port->adminStatus);
  switch(lldp_port->tx.state)
  {
    case TX_LLDP_INITIALIZE:
    {
    	lldp_port->tx.currstate = lldp_port->tx.state;
			/*one more check for port enabled */
			if(((lldp_port->adminStatus == enabledRxTx) ||
         (lldp_port->adminStatus == enabledTxOnly))
            && (lldp_port->portEnabled == TRUE)) 
			{	  
	  		txChangeToState(lldp_port, TX_IDLE);
      	lldp_port->tx.initpacket = TRUE; //send a pkt in idle state 
			}
     }
		break;
    case TX_IDLE:
    {
    	lldp_port->tx.currstate = lldp_port->tx.state;
      if(lldp_port->portEnabled == FALSE)
		  {
        txChangeToState(lldp_port, TX_LLDP_INITIALIZE);           
        break;
      }
			/* It's time to send a shutdown frame.This condition should be checked 
       * first as TX FSM is no longer available and fulfilment of other cond 
       * ll not ve any value*/
			if((lldp_port->adminStatus == disabled) ||
         (lldp_port->adminStatus == enabledRxOnly)) 
       {
	  		txChangeToState(lldp_port, TX_SHUTDOWN_FRAME);
	  	  break;
			}
	    // It's time to send a frame...
	    if((lldp_port->tx.timers.txDelayWhile == 0) && 
	       ((lldp_port->tx.timers.txTTR == 0)||
          (lldp_port->tx.somethingChangedLocal == TRUE)
          ||(lldp_port->tx.medFastStart != 0))) 
	    {
	      txChangeToState(lldp_port, TX_INFO_FRAME);
	    }

    }
    break;
    case TX_SHUTDOWN_FRAME:
    {
    	lldp_port->tx.currstate = lldp_port->tx.state;
      if(lldp_port->portEnabled == FALSE ||
         lldp_port->tx.timers.txShutdownWhile == 0)
		    txChangeToState(lldp_port, TX_LLDP_INITIALIZE);           
    }
    break;
    case TX_INFO_FRAME:
    {	
      lldp_port->tx.currstate = lldp_port->tx.state;
      if(lldp_port->portEnabled == FALSE)
		  {
        txChangeToState(lldp_port, TX_LLDP_INITIALIZE);           
        break;
      }
	    txChangeToState(lldp_port, TX_IDLE);
     }
     break;
    default:
      break;
    }
}



/******************************************************************
*  Function Name    : txStatemachineRun
*  Description      : This fn performs actions based on the current state of
*		                  Tx FSM	
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void txStatemachineRun(struct lldp_port *lldp_port)
{
  PRINT("\n IN-> txStatemachineRun ");
  /*run globalstatemcrun once more*/ 
 do{
  switch(lldp_port->tx.state)
  {
    case TX_LLDP_INITIALIZE:
    {
      if(lldp_port->tx.currstate == -1 || lldp_port->portEnabled == FALSE)
      {
       	 tx_do_tx_lldp_initialize(lldp_port);
      }
      txGlobalStatemachineRun(lldp_port);
    }break;
    case TX_IDLE:      
    {
      if(lldp_port->tx.currstate != lldp_port->tx.state)
	tx_do_tx_idle(lldp_port);
      txGlobalStatemachineRun(lldp_port);
    }break;
    case TX_SHUTDOWN_FRAME:
    {
      if(lldp_port->tx.currstate != lldp_port->tx.state)
	tx_do_tx_shutdown_frame(lldp_port);
      txGlobalStatemachineRun(lldp_port);
    }break;
    case TX_INFO_FRAME:
    {
      if(lldp_port->tx.currstate != lldp_port->tx.state)
	tx_do_tx_info_frame(lldp_port);
      txGlobalStatemachineRun(lldp_port);
    }break;
    default:
      break;
    }
  }while(lldp_port->tx.currstate != lldp_port->tx.state);
}

/******************************************************************
*  Function Name    : tx_do_tx_lldp_initialize
*  Description      : This function performs the actions when TX FSM is 
*		      in TX_LLDP_INITIALISE state.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void tx_do_tx_lldp_initialize(struct lldp_port *lldp_port)
{
  PRINT("\nIN->tx_do_tx_lldp_initialize\n");
  txInitializeLLDP(lldp_port);

  /*Do LLDP MED intiliazation*/
  struct LldpConfigMib ConfigMib;

  getMibData(localConfig,lldp_port->uiPortIndex ,NULL,(void*)&ConfigMib);

  lldp_port->medTransmitEnabled = TRUE; /*always true for an endpoint device*/
  lldp_port->tx.medFastStart=ConfigMib.medFastStartRepeatCount;   
 
  //If coming here because of PORT_ENABLED=FALSE stop all timers if started.
   IFX_LLDP_StopTimer(lldp_port->tx.unInterval); 
   IFX_LLDP_StopTimer(lldp_port->tx.unDelay);
   IFX_LLDP_StopTimer(lldp_port->tx.unShutDown);
	
  PRINT("\nOUT->tx_do_tx_lldp_initialize\n");
}

/******************************************************************
*  Function Name    : tx_do_tx_idle
*  Description      : This function performs the actions when TX FSM is 
*		      in IDLE state.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void tx_do_tx_idle(struct lldp_port *lldp_port) {
  //LLDP_FUNCTION_DEBUG;
  PRINT("\nIN->tx_do_tx_idle\n");

  /*get timer values from MIB and start them */
  struct LldpConfigMib ConfigMib;
  x_IFX_LLDP_TimerCallBackParam xLLDPTimerCB;

  getMibData(localConfig,lldp_port->uiPortIndex ,NULL,(void*)&ConfigMib);

  //txTTL-TTL of local MIB.only for remote entity
  //txTTR-interval b/w 2 LLDP frames
  //txDelayWhile-timer for changes in Local MIB.

  lldp_port->tx.txTTL = min(65535, 
    (ConfigMib.msgTxInterval * ConfigMib.msgTxHold));

  /* Check txTTR whether it was expired or not.We ve to check only this one 
     because txDelayWhile has to expire for us to come back here aft sending 
     a pkt.But txTTR can still be active.*/
  //its not expired.So first stop it then update the new value.
  if(lldp_port->tx.timers.txTTR != 0) 
    IFX_LLDP_StopTimer(lldp_port->tx.unInterval); 

  lldp_port->tx.timers.txTTR = ConfigMib.msgTxInterval;
  lldp_port->tx.somethingChangedLocal = FALSE;
  lldp_port->tx.timers.txDelayWhile = ConfigMib.txDelay;
  if(lldp_port->tx.medFastStart != 0)
  	lldp_port->tx.timers.txDelayWhile = 1;
  
  /* send an init LLDP pkt here.only 1st time.its here
   *because only when adminstatus becomes TxOnly we come to IDLE state*/
    if(lldp_port->tx.initpacket == TRUE) 
    {
      tx_do_tx_info_frame(lldp_port); 
      lldp_port->tx.initpacket = FALSE;
    }

  /*start all timers here. */  
  xLLDPTimerCB.unIndex = TX_LLDP_FRAME_INTERVAL_TIMEOUT;
  xLLDPTimerCB.ucPort = lldp_port->uiPortIndex;
  xLLDPTimerCB.uiMsgType = LLDP_LOCAL_TIMER_MSG;
  IFX_LLDP_StartTimer(lldp_port->tx.timers.txTTR * 1000000,
    &xLLDPTimerCB,&(lldp_port->tx.unInterval));  //start for txTTR
  
  xLLDPTimerCB.unIndex = TX_LLDP_LOCAL_MIB_CHANGES_TIMEOUT;
  xLLDPTimerCB.ucPort = lldp_port->uiPortIndex;
  xLLDPTimerCB.uiMsgType = LLDP_LOCAL_TIMER_MSG;
  IFX_LLDP_StartTimer(lldp_port->tx.timers.txDelayWhile * 1000000,&xLLDPTimerCB,
    &(lldp_port->tx.unDelay));  //start for txDelayWhile
  PRINT("\nOUT->tx_do_tx_idle\n");
}

/******************************************************************
*  Function Name    : tx_do_tx_shutdown_frame
*  Description      : This function performs the actions when TX FSM
*		      is in TX_SHUTDOWN_FRAME state. 
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void tx_do_tx_shutdown_frame(struct lldp_port *lldp_port) {
  //LLDP_FUNCTION_DEBUG;
  x_IFX_LLDP_TimerCallBackParam xLLDPTimerCB;

  struct LldpConfigMib ConfigMib;
  
 /*Stop all the Timers started in IDLE because TX no longer exists*/
  IFX_LLDP_StopTimer(lldp_port->tx.unInterval);
  IFX_LLDP_StopTimer(lldp_port->tx.unDelay);

  /*Send a shutdown frame*/
  mibConstrShutdownLLDPDU(lldp_port);
  txFrame(lldp_port);
 
  getMibData(localConfig,lldp_port->uiPortIndex ,NULL,(void*)&ConfigMib);
 /*This indicates the time for which LLDP entity ll be disabled*/
  lldp_port->tx.timers.txShutdownWhile = ConfigMib.reinitDelay;  
  /*Start timer for Tx Shutdown */
  xLLDPTimerCB.unIndex = TX_LLDP_SHUTDOWN_TIMEOUT;
  xLLDPTimerCB.ucPort = lldp_port->uiPortIndex;
  xLLDPTimerCB.uiMsgType = LLDP_LOCAL_TIMER_MSG;
  IFX_LLDP_StartTimer(
    lldp_port->tx.timers.txShutdownWhile * 1000000,
    &xLLDPTimerCB,&(lldp_port->tx.unShutDown));  //start for txDelayWhile
}

/******************************************************************
*  Function Name    : tx_do_tx_info_frame
*  Description      : This function performs the actions when TX FSM
*                     is in TX_INFO_FRAME state.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void tx_do_tx_info_frame(struct lldp_port *lldp_port) 
{
  mibConstrInfoLLDPDU(lldp_port);
  txFrame(lldp_port);
  if(lldp_port->tx.medFastStart != 0)
      lldp_port->tx.medFastStart--;
}

/******************************************************************
*  Function Name    : min
*  Description      : This function returns the min of 2 values
*  Input Values     : two int values 
*  Output Values    : none
*  Return Value     : minimum value
*  Notes            :
*********************************************************************/

STATIC uint16 min(uint16 value1, uint16 value2)
{
  if(value1 < value2)
    {
      return value1;
    }

  return value2;
}


