/**************************************************************************
**   FILE NAME		: ifx_lldp_cm.c
**   PROJECT		: Inca-Ip
**   MODULES	    : LLDP
**   SRC VERSION	: V1.0
**
**   DATE			:12-04-2007
**   AUTHOR			:Supreet M S, Sanat B, Awanish G, Pavan G M
**   DESCRIPTION	:Source file 	
**   COMPILER		:GCC
**   REFERENCE		:Coding Guideline Version 2.0
**							
**   COPYRIGHT      :Copyright  2004
**                   Infineon Technologies AG
**                   St. Martin Strasse 53; 81669 Mnchen, Germany
**   DESCLAIMER     :Any use of this Software is subject to the conclusion 
**                   of a respective License Agreement.
**                   Without such a License Agreement no rights to the 
**         			 Software are granted. 
**
**  Version Control Section 
**   $Author$
**   $Date$
**   $Revisions$
**   $Log$       Revision history
***********************************************************************/

#include <errno.h>
#include <sys/sem.h>
#include <sys/shm.h>

#include "ifx_common_defs.h"
#include "ifx_lldp_config.h"
#include "ifx_os.h"
#include "ifx_lldp_cm_defs.h"
#include "ifx_lldp_cm.h"

#define IFX_LLDP_CM_SHARED_MEM_KEY 12407
#define IFX_LLDP_CM_PORT_SHARED_MEM_KEY 12577
#define IFX_LLDP_CM_SEMAPHORE_KEY 70421

/*****************************************************************************
        Global Variables
*****************************************************************************/ 
int32  viCmReadFd;
int32  viCmSemId = 0, viCmShmId=0, viCmPortsShmId=0;
int32 viCmWriteFd;

/* Shared memory pointer */
x_IFX_LLDP_DB *vpxSharedMemPtr = NULL;
x_IFX_LLDP_PortCfg *pxSheardMemPortPtr = NULL;/*for number of ports*/

uint32 vxInfoTypeSizes[IFX_CM_MAX_INFOTYPES] = {
  sizeof(x_IFX_LLDP_ConfigCfg),
  sizeof(x_IFX_LLDP_TlvSelectionCfg),
  sizeof(x_IFX_LLDP_StatsCfg),
  sizeof(x_IFX_LLDP_LocalSystemCfg),
  sizeof(x_IFX_LLDP_RemoteSystemCfg),
  sizeof(x_IFX_LLDP_RemoteSystemCfg),
  sizeof(x_IFX_LLDP_RemoteSystemCfg),
  sizeof(x_IFX_LLDP_RemoteSystemCfg),
  sizeof(x_IFX_LLDP_MiscellaneousCfg)};

char8 IFX_LLDP_GetPortArrayIndex(uchar8 ucPortIndex)
{
	uchar8 i=0;
 char8 cIndex= -1;
    PRINT("IFX_LLDP_GetPortIndex : port index %d\n",ucPortIndex);
	if(vpxSharedMemPtr == NULL)
	{
		PRINT("[ERROR] IFX_LLDP_GetPortIndex : LLDP module not inited\n");
		return -1;
	}
	
	IFX_LLDP_CM_LockCfg();
    PRINT("IFX_LLDP_GetPortIndex : vpxSharedMemPtr->ucNumPorts %d\n",
      vpxSharedMemPtr->ucNumPorts);
	for(i=0;i<vpxSharedMemPtr->ucNumPorts;i++)
	{
		if(((pxSheardMemPortPtr)+i)->ucPortType == ucPortIndex)
		{
			/*that port is already inited so send the index,
			may be wants to over write port info*/			
			cIndex =i;		
			break;
		}
	}
	IFX_LLDP_CM_UnLockCfg();

	return cIndex;
}

/*****************************************************************************
*  Function Name        :   IFX_CM_GetCfgData
*  Description          :   This function gets the values of all TLV's from either 
*                           local or remote system MIB based on the input parameters
*  Input Values         :   InterfaceId,ucInfoType and pucMsap
*  Output Values        :   Data
*  Return Value         :   IFX_CM_SUCCESS - On Success
*                           IFX_CM_FAIL - On Failure
*  Notes                :
*****************************************************************************/
PUBLIC char8 IFX_LLDP_CM_GetCfgData(IN uchar8 ucInterfaceId,
                               IN uchar8 ucInfoType,
							                 IN uchar8 *pucMsap,
                               OUT void *pxData)
{
  void *ptr=NULL;
  uint32 uiIndex;
  uint8 uiRet=IFX_CM_FAIL;
  //TODO:Actual OS Lock with semID should be used here
  char8 cInterfaceIdTemp = IFX_LLDP_GetPortArrayIndex(ucInterfaceId);
  if (cInterfaceIdTemp == -1)
  {
    PRINT("********cInterfaceIdTemp: -1\n");
    return uiRet;
  }
  ucInterfaceId = cInterfaceIdTemp;
  PRINT("IFX_LLDP_CM_GetCfgData : ucInterfaceId - %d\n",ucInterfaceId);
  IFX_LLDP_CM_LockCfg(); 
  IFX_LLDP_CM_GetCfgPtr(ucInterfaceId,ucInfoType,&ptr);
  switch(ucInfoType)
  {
    case IFX_CM_LLDP_CONFIG_CFG:
	case IFX_CM_TLV_SELECTION_CONFIG_CFG:
	case IFX_CM_LLDP_STATS_CONFIG_CFG:
	case IFX_CM_LOCAL_MIB_CFG:
	case IFX_CM_LLDP_MISCELLANEOUS_CFG:
	  memcpy(pxData,ptr,vxInfoTypeSizes[ucInfoType]);
	  uiRet=IFX_CM_SUCCESS;
	  break;
	case IFX_CM_REMOTE_MIB_CFG_MSAP:
	{
  x_IFX_LLDP_RemoteSystemCfg *pRemoteSystem = (x_IFX_LLDP_RemoteSystemCfg *)ptr;
  PRINT("********IFX_LLDP_CM_GetCfgData :pRemoteSystem->uiNoOfEntries %d\n",pRemoteSystem->uiNoOfEntries);
      for (uiIndex =0;uiIndex< pRemoteSystem->uiNoOfEntries;uiIndex++)
	  {
  PRINT("@@@@@@IFX_LLDP_CM_GetCfgData : pucMsap %s\n",pucMsap);
  PRINT("@@@@@@IFX_LLDP_CM_GetCfgData : pRemoteSystem->axRemoteMibData Msap %s\n",pRemoteSystem->axRemoteMibData[uiIndex].ucMsap);
         if(!strcmp(pucMsap,pRemoteSystem->axRemoteMibData[uiIndex].ucMsap))
		 {
  PRINT("********IFX_LLDP_CM_GetCfgData : Match Found\n");
		   memcpy(pxData,&(pRemoteSystem->axRemoteMibData[uiIndex].xRemoteMib),sizeof(x_IFX_LLDP_TlvData));
	       uiRet=IFX_CM_SUCCESS;
		 }
	  }
	  break;
	}
	case IFX_CM_REMOTE_MIB_CFG_MAC:
	{
	  x_IFX_LLDP_RemoteSystemCfg *pRemoteSystem = (x_IFX_LLDP_RemoteSystemCfg *)ptr;
      for (uiIndex =0;uiIndex< pRemoteSystem->uiNoOfEntries;uiIndex++)
	  {
         if(memcmp(pucMsap,&(pRemoteSystem->axRemoteMibData[uiIndex].uiaMacAddress),
		     sizeof( pRemoteSystem->axRemoteMibData[uiIndex].uiaMacAddress))==0)
		 {
		   memcpy(pxData,&(pRemoteSystem->axRemoteMibData[uiIndex].xRemoteMib),sizeof(x_IFX_LLDP_TlvData));
	           uiRet=IFX_CM_SUCCESS;
		 }
	  }
	  break;
	}

	case IFX_CM_LLDP_RAW_FRAME:
	{
	  x_IFX_LLDP_RemoteSystemCfg *pRemoteSystem = (x_IFX_LLDP_RemoteSystemCfg *)ptr;
      for (uiIndex =0;uiIndex< pRemoteSystem->uiNoOfEntries;uiIndex++)
	  {
         if(memcmp(pucMsap,&(pRemoteSystem->axRemoteMibData[uiIndex].uiaMacAddress),
		   sizeof(pRemoteSystem->axRemoteMibData[uiIndex].uiaMacAddress))==0)
		 {
		   memcpy(pxData,&(pRemoteSystem->axRemoteMibData[uiIndex].ucFrame),IFX_LLDP_MAX_FRAME_SIZE);
	       uiRet=IFX_CM_SUCCESS;
	    }
	  }
	  break;
	}
	default:
	  break;
  }
  IFX_LLDP_CM_UnLockCfg();
  return uiRet;
}
/*****************************************************************************
*  Function Name        :   IFX_CM_SetCfgData 
*  Description          :   This function either sets the values of all TLV's 
*                           of the local system MIB or adds a new remote system MIB 
*							to the remote system MIB table based on the input parameters
*  Input Values         :   InterfaceId,ucInfoType,pucMsap and Data
*  Output Values        :   
*  Return Value         :   IFX_CM_SUCCESS - On Success
*                           IFX_CM_FAIL - On Failure
*  Notes                :
*****************************************************************************/
PUBLIC char8 IFX_LLDP_CM_SetCfgData(IN uchar8 ucInterfaceId,
                               IN uchar8 ucInfoType,
                               IN uchar8 *pucMsap,
                               IN void *pxData)
{
  void *ptr=NULL;
  x_IFX_LLDP_MiscellaneousCfg *miscPtr=NULL;
  uint32 uiIndex;
  uint8 uiRet=IFX_CM_FAIL;
  char8 cInterfaceIdTemp = IFX_LLDP_GetPortArrayIndex(ucInterfaceId);
  if (cInterfaceIdTemp == -1)
     return uiRet;
  ucInterfaceId = cInterfaceIdTemp;
  PRINT("IFX_LLDP_CM_SetCfgData : ucInterfaceId %d\n",ucInterfaceId);
  IFX_LLDP_CM_LockCfg(); 
  IFX_LLDP_CM_GetCfgPtr(ucInterfaceId,ucInfoType,&ptr);
  switch(ucInfoType)
  {
    case IFX_CM_LLDP_CONFIG_CFG:
	case IFX_CM_TLV_SELECTION_CONFIG_CFG:
	case IFX_CM_LLDP_STATS_CONFIG_CFG:
	case IFX_CM_LOCAL_MIB_CFG:
	case IFX_CM_LLDP_MISCELLANEOUS_CFG:
	  memcpy(ptr,pxData,vxInfoTypeSizes[ucInfoType]);
	  uiRet=IFX_CM_SUCCESS;
	  break;
	case IFX_CM_REMOTE_MIB_CFG_MSAP:
	{
	  x_IFX_LLDP_RemoteSystemCfg *pRemoteSystem = (x_IFX_LLDP_RemoteSystemCfg *)ptr;
	  uint8 uiMsapExist =0;
      for (uiIndex =0;uiIndex< pRemoteSystem->uiNoOfEntries;uiIndex++)
	  {
         if(!strcmp(pucMsap,pRemoteSystem->axRemoteMibData[uiIndex].ucMsap))
		 {
		   memcpy(&(pRemoteSystem->axRemoteMibData[uiIndex].xRemoteMib),pxData,sizeof(x_IFX_LLDP_TlvData));
		   uiMsapExist =1;
	       uiRet=IFX_CM_SUCCESS;
		 }
	  }
	  if (uiMsapExist == 0)
	  {
	    if(pRemoteSystem->uiNoOfEntries < IFX_LLDP_MAX_REMOTE_MIB_PER_PORT)
		{		  
	      memcpy(&(pRemoteSystem->axRemoteMibData[pRemoteSystem->uiNoOfEntries].xRemoteMib)
		          ,pxData,sizeof(x_IFX_LLDP_TlvData));	  
	      strcpy(pRemoteSystem->axRemoteMibData[uiIndex].ucMsap,pucMsap);
		  pRemoteSystem->uiNoOfEntries++;
          IFX_LLDP_CM_GetCfgPtr(ucInterfaceId,IFX_CM_LLDP_MISCELLANEOUS_CFG,(void *)&miscPtr);
		  miscPtr->uiCurrentRemoteEntries +=1;
		}
	  }
	  break;
	}
	case IFX_CM_LLDP_RAW_FRAME:
	{
	  x_IFX_LLDP_RemoteSystemCfg *pRemoteSystem = (x_IFX_LLDP_RemoteSystemCfg *)ptr;
	  uint8 uiMsapExist =0;
      for (uiIndex =0;uiIndex< pRemoteSystem->uiNoOfEntries;uiIndex++)
	  {
        if(memcmp(pucMsap,&(pRemoteSystem->axRemoteMibData[uiIndex].uiaMacAddress),
		   sizeof(pRemoteSystem->axRemoteMibData[uiIndex].uiaMacAddress))==0)
		{
		  memcpy(&(pRemoteSystem->axRemoteMibData[uiIndex].ucFrame),pxData,IFX_LLDP_MAX_FRAME_SIZE);
		  uiMsapExist =1;
	      uiRet=IFX_CM_SUCCESS;
	    }
	  }
	  if (uiMsapExist == 0)
	  {
	    if(pRemoteSystem->uiNoOfEntries < IFX_LLDP_MAX_REMOTE_MIB_PER_PORT)
		{
	      memcpy(&(pRemoteSystem->axRemoteMibData[uiIndex].uiaMacAddress),pucMsap,
		    sizeof(pRemoteSystem->axRemoteMibData[uiIndex].uiaMacAddress));
	      memcpy(&(pRemoteSystem->axRemoteMibData[uiIndex].ucFrame),pxData,IFX_LLDP_MAX_FRAME_SIZE);
		 // pRemoteSystem->uiNoOfEntries++;
		}
	  }
	  break;
	}
  case IFX_CM_REMOTE_MAC:
  {
    x_IFX_LLDP_RemoteSystemCfg *pRemoteSystem = (x_IFX_LLDP_RemoteSystemCfg *)ptr;
	  uint8 uiMsapExist =0;
      for (uiIndex =0;uiIndex< pRemoteSystem->uiNoOfEntries;uiIndex++)
	  {
        if(memcmp(pucMsap,&(pRemoteSystem->axRemoteMibData[uiIndex].uiaMacAddress),
		   sizeof(pRemoteSystem->axRemoteMibData[uiIndex].uiaMacAddress))==0)
		{
		  uiMsapExist =1;
	      uiRet=IFX_CM_SUCCESS;
	    }
	  }
	  if (uiMsapExist == 0)
	  {
	    if(pRemoteSystem->uiNoOfEntries < IFX_LLDP_MAX_REMOTE_MIB_PER_PORT)
		{
	      memcpy(&(pRemoteSystem->axRemoteMibData[uiIndex].uiaMacAddress),pucMsap,
		    sizeof(pRemoteSystem->axRemoteMibData[uiIndex].uiaMacAddress));
		}
	  }
	  break;
  
  }
	default:
	  break;
  }
  IFX_LLDP_CM_UnLockCfg();
  return uiRet;

}

/*****************************************************************************
*  Function Name        :   IFX_CM_DeleteCfgData
*  Description          :   This function deletes a remote system MIB entry 
*                           from the Remote system MIB table
*  Input Values         :   InterfaceId and pucMsap
*  Output Values        :   
*  Return Value         :   IFX_CM_SUCCESS - On Success
*                           IFX_CM_FAIL - On Failure
*  Notes                :
*****************************************************************************/
PUBLIC char8 IFX_LLDP_CM_DeleteCfgData(IN uchar8 ucInterfaceId,
                                       IN uchar8 ucInfoType,
                                       IN uchar8 *pucMsap)
{
  x_IFX_LLDP_RemoteSystemCfg *pRemoteSystem;
  x_IFX_LLDP_MiscellaneousCfg *miscPtr=NULL;
  uint32 uiIndex,uiDelIndex,uiMatchFound=0;
  uint8 uiRet=IFX_CM_FAIL;
  char8 cInterfaceIdTemp = IFX_LLDP_GetPortArrayIndex(ucInterfaceId);
  if (cInterfaceIdTemp == -1)
     return uiRet;
  ucInterfaceId = cInterfaceIdTemp;
  IFX_LLDP_CM_LockCfg(); 
  IFX_LLDP_CM_GetCfgPtr(ucInterfaceId,IFX_CM_REMOTE_MIB_CFG_MAC,(void *)&pRemoteSystem);
  if (ucInfoType == IFX_CM_REMOTE_MIB_CFG_MSAP)
  {
    for (uiIndex =0;uiIndex< pRemoteSystem->uiNoOfEntries;uiIndex++)
    {
      if(!strcmp(pucMsap,pRemoteSystem->axRemoteMibData[uiIndex].ucMsap))
	    {
	      uiDelIndex = uiIndex;
	      uiMatchFound =1;
	      uiRet=IFX_CM_SUCCESS;
	      break;
	    }
    }
  }
  else if (ucInfoType == IFX_CM_REMOTE_MIB_CFG_MAC)
  {
    for (uiIndex =0;uiIndex< pRemoteSystem->uiNoOfEntries;uiIndex++)
    {
      if(memcmp(pucMsap,&(pRemoteSystem->axRemoteMibData[uiIndex].uiaMacAddress),
	       sizeof(pRemoteSystem->axRemoteMibData[uiIndex].uiaMacAddress))==0)
	  {
	    uiDelIndex = uiIndex;
	    uiMatchFound =1;
	    uiRet=IFX_CM_SUCCESS;
	    break;
	  }
    }
  }	
  if(uiMatchFound == 1)
  {
    if (uiDelIndex == (pRemoteSystem->uiNoOfEntries -1))
    {
      memset(&(pRemoteSystem->axRemoteMibData[uiDelIndex]),'\0',sizeof(x_IFX_LLDP_RemoteSystem)); 
    } else {
      memcpy(&(pRemoteSystem->axRemoteMibData[uiDelIndex]),&(pRemoteSystem->axRemoteMibData[uiDelIndex+1]),
  	    sizeof(x_IFX_LLDP_RemoteSystem)*((pRemoteSystem->uiNoOfEntries-1)-uiDelIndex));
    }
    pRemoteSystem->uiNoOfEntries--;
    IFX_LLDP_CM_GetCfgPtr(ucInterfaceId,IFX_CM_LLDP_MISCELLANEOUS_CFG,(void *)&miscPtr);
	miscPtr->uiCurrentRemoteEntries -=1;
        PRINT("IFX_LLDP_CM_DeleteCfgData : Entry found and deleted\n");
  }
  IFX_LLDP_CM_UnLockCfg();
  return uiRet;
}

/*****************************************************************************
*  Function Name        :   IFX_LLDP_CM_SHM_Init
*  Description          :   This function creates shared mem required 
							for LLDP module.
*                           
*  Input Values         :   None 
*  Output Values        :   None
*  Return Value         :   0 - On Success
*                           -1 - On Failure
*  Notes                :
*****************************************************************************/
char8 IFX_LLDP_CM_SHM_Init(uchar8 ucNumPorts)
{

  int32 iRet=0;
  ux_IFX_OS_SEMUN uxArg={0};
  /* Create Shared memory for outer structure*/
  if ((viCmShmId =
       IFX_OS_ShmCreate(IFX_LLDP_CM_SHARED_MEM_KEY,
                        sizeof(x_IFX_LLDP_DB),
                        IPC_CREAT |IPC_EXCL|0666))
       < 0)
  {
    /*check if shm is created already*/
    if (errno == EEXIST)
    {
      PRINT("\n@@@@@ errno returned is EEXIST\n");		  
      iRet = errno;/*store that flag saying it exists*/
    }
  }
  /*anyway if it's there or not u have to get shmid using shm create*/
  if ((viCmShmId =
       IFX_OS_ShmCreate(IFX_LLDP_CM_SHARED_MEM_KEY,
       sizeof(x_IFX_LLDP_DB),
       IPC_CREAT|0666))
       < 0)
  {
    perror("[ERROR] IFX_LLDP_CM_SHM_Init : Error in creating shared memory is - "); 
    return -1;
  }
     
  /* Attach the shared memory to the process*/  
  if ((vpxSharedMemPtr = IFX_OS_ShmAttach(viCmShmId,NULL,0)) == NULL)
  {
     perror("[ERROR] IFX_LLDP_CM_SHM_Init : Error in attaching shared memory is"); 
     return -1;
  }
  if(iRet == 0)/*shm is created for the first time so set it to zero*/ 
  {
     memset(vpxSharedMemPtr,0,sizeof(x_IFX_LLDP_DB));
  }

  /* ///////////Create Shared memory for port structure based on number of ports/////// */
  if (iRet == EEXIST)
  {
    ucNumPorts=1;
  }
  if ((viCmPortsShmId =
       IFX_OS_ShmCreate(IFX_LLDP_CM_PORT_SHARED_MEM_KEY,
                        (sizeof(x_IFX_LLDP_PortCfg)*ucNumPorts),
                        IPC_CREAT |0666))
       < 0)
  {
     perror("[ERROR] IFX_LLDP_CM_SHM_Init : Error creating port shared memory is - ");
     return -1;
  }
		  
  /* Attach the shared memory*/  
  if ((pxSheardMemPortPtr = IFX_OS_ShmAttach(viCmPortsShmId,NULL,0)) == NULL)
  {
     perror("[ERROR] IFX_LLDP_CM_SHM_Init : Error in attaching port shared memory is - "); 
     return -1;
  }

  if(iRet == 0)/*shm is created for the first time so set it to zero*/
  {
    memset(pxSheardMemPortPtr,0,(sizeof(x_IFX_LLDP_PortCfg)*ucNumPorts));
  }

  /* Create Semaphore */
  if (( viCmSemId = IFX_OS_SemCreate(IFX_LLDP_CM_SEMAPHORE_KEY, 
				1,IPC_CREAT | IPC_EXCL |0666)) < 0){
	if( errno == EEXIST ){
		/* Semaphore already created and initialised; just get a handle to exisiting semaphore */
  		if (( viCmSemId = IFX_OS_SemCreate(IFX_LLDP_CM_SEMAPHORE_KEY, 1,IPC_CREAT |0666)) < 0){
     			perror("[ERROR] IFX_LLDP_CM_SHM_Init : Error in reating semaphore is - "); 
     			return -1;
		}
	}
  }else{
	/* First time creation of the semaphore initialise the semaphore */
  
  	/* Initilize Sempahore */
  	uxArg.val = 1;
  	if (IFX_OS_SemCtl(viCmSemId,0,SETVAL,uxArg) < 0) {
    		perror("[ERROR] IFX_LLDP_CM_SHM_Init : Error in initilizing semaphore is - "); 
     		return -1;
  	}
  }

  PRINT("\nIFX_LLDP_CM_SHM_Init OK vpxSharedMemPtr %X pxSheardMemPortPtr %X viCmSemId %X\n", vpxSharedMemPtr, pxSheardMemPortPtr, viCmSemId );

	return iRet;
}
/*****************************************************************************
*  Function Name        :   IFX_LLDP_CM_SHM_Shut
*  Description          :   This function releases the shared mem created 
							for LLDP module.
*                           
*  Input Values         :   None 
*  Output Values        :   None
*  Return Value         :   0 - On Success
*                           -1 - On Failure
*  Notes                :
*****************************************************************************/
PUBLIC char8 IFX_LLDP_CM_SHM_Shut(void)
{
   char8 cRet = 0;
   
   /* Closing Shared meory */
   if ( (cRet= IFX_OS_ShmCtl(viCmShmId, IPC_RMID, NULL)) == IFX_CM_FAIL)
   {
     return IFX_CM_FAIL;
   }
   if ( (cRet= IFX_OS_ShmCtl(viCmPortsShmId, IPC_RMID, NULL)) == IFX_CM_FAIL)
   {
     return IFX_CM_FAIL;
   }
  
   /*closing Semaphore */
   if ( (cRet= IFX_OS_RemoveSem(viCmSemId,0)) == IFX_CM_FAIL)
   {
       return IFX_CM_FAIL;
   }
   
   return cRet;

}
/*****************************************************************************
*  Function Name        :   IFX_LLDP_CM_Init
*  Description          :   This function creates FIFO required 
							for LLDP module.
*                           
*  Input Values         :   None 
*  Output Values        :   None
*  Return Value         :   0 - On Success
*                           -1 - On Failure
*  Notes                :
*****************************************************************************/
char8 IFX_LLDP_CM_Init(void)
{

  int32 viCmReadFdTemp;
  /*create a FIFO for LLDP*/
  if( IFX_OS_CreateFifo(IFX_LLDP_FIFO) < 0 )
  {
	  perror("[ERROR]IFX_OS_CreateFifo : ");
  }

  /* open FIFO for read */

  viCmReadFdTemp= IFX_OS_OpenFifo(IFX_LLDP_FIFO, O_RDONLY|O_NONBLOCK );
  if (viCmReadFdTemp < 0)
  {
      perror("[ERROR] IFX_OS_OpenFifo : ReadFd is");
      return -1;
  }

  /* open FIFO for write */

  viCmWriteFd= IFX_OS_OpenFifo(IFX_LLDP_FIFO, O_WRONLY );
  if (viCmWriteFd < 0)
  {
      perror("[ERROR] IFX_OS_OpenFifo :WriteFd is");
      return -1;
  }
  IFX_OS_Close(viCmReadFdTemp);
  return 0;
}


/*****************************************************************************
*  Function Name        :   IFX_LLDP_CMFIFO_Write
*  Description          :   This function writes in to CMFIFO
*                           
*  Input Values         :   None 
*  Output Values        :   None
*  Return Value         :   0 - On Success
*                           -1 - On Failure
*  Notes                :
*****************************************************************************/
char8 IFX_LLDP_CMFIFO_Write(void *Buffer, uint32 uiSize)
{
	if(viCmWriteFd == 0)
	{
		PRINT("[ERROR] IFX_LLDP_CMFIFO_Write : CM not inited\n");
		return -1;
	}

    if(write(viCmWriteFd,Buffer,uiSize) < 0)
	{
		perror("[ERROR] IFX_LLDP_CMFIFO_Write : error  - ");
		return -1;
	}
	return 0;
}

/*****************************************************************************
*  Function Name        :   IFX_LLDP_CMFIFO_Read
*  Description          :   This function Reads from CMFIFO
*                           
*  Input Values         :   None 
*  Output Values        :   None
*  Return Value         :   0 - On Success
*                           -1 - On Failure
*  Notes                :
*****************************************************************************/
char8 IFX_LLDP_CMFIFO_Read(void *Buffer, uint32 uiSize)
{
	if(viCmReadFd == 0)
	{
		PRINT("[ERROR] IFX_LLDP_CMFIFO_Read : CM not inited\n");
		return -1;
	}

    if(read(viCmReadFd,Buffer,uiSize) < 0)
	{
		perror("[ERROR] IFX_LLDP_CMFIFO_Read : error - ");
		return -1;
	}
	return 0;
}

/*****************************************************************************
*  Function Name        :   IFX_LLDP_CM_SemLock
*  Description          :   This function Locks the LLDP Semapore
*                           
*  Input Values         :   None 
*  Output Values        :   None
*  Return Value         :   0 - On Success
*                           -1 - On Failure
*  Notes                :
*****************************************************************************/
char8 IFX_LLDP_CM_LockCfg(void)
{
	if(viCmSemId == -1)
	{
		PRINT("IFX_LLDP_CM_LockCfg : CM not inited\n");
		return -1;
	}
   return IFX_OS_LOCK(viCmSemId);
}

/*****************************************************************************
*  Function Name        :   IFX_LLDP_CM_SemUnLock
*  Description          :   This function UnLocks the LLDP Semapore
*                           
*  Input Values         :   None 
*  Output Values        :   None
*  Return Value         :   0 - On Success
*                           -1 - On Failure
*  Notes                :
*****************************************************************************/
char8 IFX_LLDP_CM_UnLockCfg(void)
{
	if(viCmSemId == -1)
	{
		PRINT("[ERROR] IFX_LLDP_CM_UnLockCfg : CM not inited\n");
		return -1;
	}
   return IFX_OS_UNLOCK(viCmSemId);
}

/*****************************************************************************
*  Function Name        :   IFX_LLDP_CM_GetCfgPtr
*  Description          :   This function returns the appropriate shared 
                            memory pointer based on Interface and InfoType
*                           
*  Input Values         :   None 
*  Output Values        :   None
*  Return Value         :   0 - On Success
*                           -1 - On Failure
*  Notes                :
*****************************************************************************/
char8 IFX_LLDP_CM_GetCfgPtr(IN uchar8 ucInterfaceId, IN uchar8 ucInfoType, void **ptr)
{
	if(viCmShmId == -1) /*cm shared memory not created that means module not inited*/
	{
		PRINT("[ERROR] IFX_LLDP_CM_GetCfgPtr : LLDP module not inited\n");
		return -1;
	}
   
   switch(ucInfoType)
   {
       case IFX_CM_LLDP_CONFIG_CFG :
	   {
		   *ptr=&(((pxSheardMemPortPtr)+ucInterfaceId)->xConfigCfg);

		   break;
	   }
	   case IFX_CM_TLV_SELECTION_CONFIG_CFG :
	   {
		   *ptr=&(((pxSheardMemPortPtr)+ucInterfaceId)->xTlvSelectionCfg);
		   
		   break;
	   }
	   case IFX_CM_LLDP_STATS_CONFIG_CFG :
	   {
		   //*ptr=&(((pxSheardMemPortPtr)+ucInterfaceId)->xStatsCfg);
		   *ptr = &(vpxSharedMemPtr->xStatsCfg);
		   
		   break;
	   }
	   case IFX_CM_LOCAL_MIB_CFG :
	   {
		   *ptr=&(((pxSheardMemPortPtr)+ucInterfaceId)->xLocalSystemCfg);
		   break;
	   }
	   case IFX_CM_REMOTE_MIB_CFG_MAC:
	   case IFX_CM_REMOTE_MIB_CFG_MSAP :
	   {
		   //*ptr=&(((pxSheardMemPortPtr)+ucInterfaceId)->xRemoteSystemCfg);
		   *ptr = &(vpxSharedMemPtr->xRemoteSystemCfg);
		   
		   break;
	   }
	   case IFX_CM_LLDP_RAW_FRAME :
	   {
		   //*ptr=&(((pxSheardMemPortPtr)+ucInterfaceId)->xRemoteSystemCfg);
		   *ptr = &(vpxSharedMemPtr->xRemoteSystemCfg);
		   
		   break;
	   }
		case IFX_CM_REMOTE_MAC :
	   {
		   //*ptr=&(((pxSheardMemPortPtr)+ucInterfaceId)->xRemoteSystemCfg);
		   *ptr = &(vpxSharedMemPtr->xRemoteSystemCfg);
		   
		   break;
	   }

	   case IFX_CM_LLDP_MISCELLANEOUS_CFG:
	   {
		   //*ptr=&(((pxSheardMemPortPtr)+ucInterfaceId)->xMiscellaneousCfg);
		   *ptr = &(vpxSharedMemPtr->xMiscellaneousCfg);
		   
		   break;
	   }
	   default :
	   {
		   PRINT("\n [ERROR] IFX_LLDP_CM_GetCfgPtr : wrong infotype\n");
	   }
   }
	return 0;
}
